package main

import (
	"context"
	"flag"
	"fmt"
	"net/http"
	"os"
	"os/signal"
	"time"

	"github.com/facebookarchive/flagenv"
	_ "github.com/joho/godotenv/autoload"
	"within.website/ln"
	"within.website/ln/ex"
	"within.website/mi/rethink"
	"within.website/x/web/useragent"
)

var (
	/*
		// Gitea
		giteaInstance = flag.String("gitea-instance", "", "Gitea instance to use")
		giteaAPIToken = flag.String("gitea-api-token", "", "Gitea API token")

		// Github
		githubToken = flag.String("github-token", "", "Github API token")
	*/

	// RethinkDB
	rethinkDBURL = flag.String("rethinkdb-url", "rethinkdb2://admin@127.0.0.1:28015/mi", "RethinkDB URL")

	// Port
	port   = flag.String("port", "5000", "HTTP port")
	domain = flag.String("domain", "mi.within.website", "domain this is being served on")
)

func main() {
	flagenv.Parse()
	flag.Parse()

	http.DefaultTransport = useragent.Transport("mi-posse", "https://"+*domain+"/.within/botinfo", http.DefaultTransport)

	ctx := context.Background()

	session, err := rethink.GetSession(*rethinkDBURL)

	mastodonClient, err := makeMastodon()
	if err != nil {
		ln.FatalErr(ctx, err)
	}

	redditBot, err := makeReddit()
	if err != nil {
		ln.FatalErr(ctx, err)
	}

	twitterClient, err := makeTwitter()
	if err != nil {
		ln.FatalErr(ctx, err)
	}

	patreonClient, err := makePatreon()
	if err != nil {
		ln.FatalErr(ctx, err)
	}

	mux := http.NewServeMux()
	pm, err := MakeMiddleware(*pasetoPublicKey, *pasetoPrivateKey, mux)
	if err != nil {
		ln.FatalErr(ctx, err)
	}

	tok, err := pm.CreateToken(
		[][2]string{
			{"from", "main"},
		},
		time.Now().AddDate(2, 0, 0),
	)
	if err != nil {
		ln.FatalErr(ctx, err)
	}

	fmt.Printf("token: %s\n", tok)

	mi := &Mi{
		session:        session,
		mastodonClient: mastodonClient,
		twitterClient:  twitterClient,
		patreonClient:  patreonClient,
		redditBot:      redditBot,
		mux:            mux,
	}
	mi.RegisterRoutes()
	mi.RegisterPusher(mi.PushDiscordWebhook)
	mi.RegisterPusher(mi.PushReddit)
	mi.RegisterPusher(mi.PushMastodon)
	mi.RegisterPusher(mi.PushTwitter)

	ctx, cancel := context.WithCancel(ctx)
	go func(cf func()) {
		var signalChan chan os.Signal
		signalChan = make(chan os.Signal, 1)
		signal.Notify(signalChan, os.Interrupt)

		<-signalChan
		cf()
	}(cancel)

	go mi.StreamMastodon(ctx)
	go mi.StreamReddit(ctx)
	go mi.StreamMastodonToTwitter(ctx)
	go mi.BlogPOSSE(ctx)

	h := &http.Server{
		Addr:    ":" + *port,
		Handler: ex.HTTPLog(pm),
	}

	go func() { ln.FatalErr(ctx, h.ListenAndServe()) }()

	<-ctx.Done()
	ln.Log(ctx, ln.Info("cleaning up"))
	time.Sleep(500 * time.Millisecond)
}
